import { Observable } from 'rxjs/Observable';
import { AuthFilterError } from '../errors/auth-filter.error';
import { PermissionResult } from '../public-api/permission-result';
import { Log } from '../internal/log';
var EndPointBuilder = /** @class */ (function () {
    function EndPointBuilder() {
    }
    EndPointBuilder.build = function (mViewset, key, routeConfig, endPointConfig, router) {
        var endpointDescription = endPointConfig.method.toUpperCase() + ' ' + routeConfig.path + endPointConfig.path
            + ' => ' + mViewset.name + '.' + key;
        Log.i(endpointDescription);
        return function (req, resp, next) {
            Log.d(endpointDescription);
            if (req.body) {
                Log.d('body: ' + JSON.stringify(req.body));
            }
            try {
                var viewset = new mViewset();
                EndPointBuilder.manageEndPointAuthFilter(req, resp, routeConfig, endPointConfig, viewset, key);
            }
            catch (e) {
                Log.e(e);
                resp.status(500);
                resp.json({
                    reason: 'Unhandled error',
                    details: e
                });
            }
        };
    };
    EndPointBuilder.manageEndPointError = function (req, resp, err) {
        if (err instanceof AuthFilterError) {
            resp.status(401);
            resp.json({
                reason: 'Invalid request authentication informations',
                details: err.message
            });
        }
        else {
            Log.e(err);
            resp.status(500);
            resp.json({
                reason: 'Unhandled error',
                details: err instanceof String ? err : err.toString()
            });
        }
    };
    EndPointBuilder.manageEndPointAuthFilter = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var user;
        var err;
        EndPointBuilder.callEndPointAuthFilter(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else {
                if (user) {
                    req.user = user;
                }
                EndPointBuilder.manageEndPointAuthPermission(req, resp, routeConfig, endPointConfig, viewset, key);
            }
        })
            .subscribe(function (iUser) {
            user = iUser;
        }, function (authFilterErr) {
            Log.e(err);
            err = authFilterErr;
        });
    };
    EndPointBuilder.callEndPointAuthFilter = function (req, routeConfig, endPointConfig) {
        var edpFilters = [];
        var authFilter = endPointConfig.authFilter || routeConfig.authFilter;
        if (authFilter) {
            Log.d(endPointConfig.authFilter ? endPointConfig.authFilter.constructor.name : 'no enpoint filter');
            Log.d(routeConfig.authFilter ? routeConfig.authFilter.constructor.name : 'no route filter');
            edpFilters.push(authFilter);
        }
        if (!edpFilters.length) {
            edpFilters = edpFilters.concat(EndPointBuilder.authFilters);
        }
        if (routeConfig.addAuthFilters) {
            edpFilters = edpFilters.concat(routeConfig.addAuthFilters);
        }
        if (endPointConfig.addAuthFilters) {
            edpFilters = edpFilters.concat(endPointConfig.addAuthFilters);
        }
        if (!edpFilters.length) {
            var u = new EndPointBuilder.userModel();
            u.isAnonymous = true;
            return Observable.from([u]);
        }
        else {
            return EndPointBuilder.scanAuthFilters(req, edpFilters);
        }
    };
    EndPointBuilder.scanAuthFilters = function (req, edpFilters, index) {
        if (index === void 0) { index = 0; }
        return edpFilters[index].filter(req, EndPointBuilder.userModel).switchMap(function (user) {
            if (!user && index < edpFilters.length - 1) {
                return EndPointBuilder.scanAuthFilters(req, edpFilters, index + 1);
            }
            else {
                return Observable.create(function (observer) {
                    observer.next(user);
                    observer.complete();
                });
            }
        });
    };
    EndPointBuilder.manageEndPointAuthPermission = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var result;
        var err;
        EndPointBuilder.callAuthPermissions(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else if (!result || result.isOk) {
                EndPointBuilder.manageEndPointPermissions(req, resp, routeConfig, endPointConfig, viewset, key);
            }
            else {
                resp.status(401);
                resp.json({
                    reason: 'Request not authenticated',
                    details: result ? result.rejectionReason : ''
                });
            }
        }).subscribe(function (permissionResult) {
            result = permissionResult;
        }, function (authPermissionErr) {
            err = authPermissionErr;
        });
    };
    EndPointBuilder.callAuthPermissions = function (req, routeConfig, endPointConfig) {
        var authPermission = endPointConfig.authPermission || routeConfig.authPermission || EndPointBuilder.defaultAuthPermission;
        if (authPermission) {
            return authPermission.check(req, EndPointBuilder.userModel);
        }
        else {
            return Observable.create(function (observer) {
                observer.next(new PermissionResult(true));
                observer.complete();
            });
        }
    };
    EndPointBuilder.manageEndPointPermissions = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var result;
        var err;
        EndPointBuilder.callPermissions(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else if (!result || result.isOk) {
                try {
                    viewset[key](req, resp);
                }
                catch (e) {
                    EndPointBuilder.manageEndPointError(req, resp, e);
                }
            }
            else {
                resp.status(403);
                resp.json({
                    reason: 'Request not authorized',
                    details: result ? result.rejectionReason : ''
                });
            }
        }).subscribe(function (permissionResult) {
            result = permissionResult;
        }, function (error) {
            err = error;
        });
    };
    EndPointBuilder.callPermissions = function (req, routeConfig, endPointConfig) {
        var permissionFilters = [];
        var permissionFilter = endPointConfig.permissionFilter || routeConfig.permissionFilter || EndPointBuilder.defaultPermissionFilter;
        if (permissionFilter) {
            permissionFilters.push(permissionFilter);
        }
        if (routeConfig.addPermissionFilters) {
            permissionFilters = permissionFilters.concat(routeConfig.addPermissionFilters);
        }
        if (endPointConfig.addPermissionFilters) {
            permissionFilters = permissionFilters.concat(endPointConfig.addPermissionFilters);
        }
        if (permissionFilters.length) {
            return EndPointBuilder.scanPermissionFilters(req, permissionFilters, 0);
        }
        else {
            return Observable.empty();
        }
    };
    EndPointBuilder.scanPermissionFilters = function (req, permissionFilters, index) {
        return permissionFilters[index].filter(req, EndPointBuilder.userModel).switchMap(function (result) {
            if (result.isOk && index < permissionFilters.length - 1) {
                return EndPointBuilder.scanPermissionFilters(req, permissionFilters, index + 1);
            }
            else {
                return Observable.create(function (observer) {
                    observer.next(result);
                    observer.complete();
                });
            }
        });
    };
    EndPointBuilder.authFilters = [];
    return EndPointBuilder;
}());
export { EndPointBuilder };
//# sourceMappingURL=endpoint-builder.js.map