import { EndPointBuilder } from './internal/endpoint-builder';
import { QueryManager } from 'ts-db-helper';
import { RouteManager } from './managers/route.manager';
import { IsAutenticatedPermission } from './security/permissions/is-authenticated.permission';
import { Log } from './internal/log';
import * as express from 'express';
import * as bodyParser from 'body-parser';
import * as http from 'http';
import 'rxjs/add/operator/finally';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/Observable/empty';
var Express = express;
// Creates and configures an ExpressJS web server.
var TsRestFrameworkApp = /** @class */ (function () {
    // Run configuration methods on the Express instance.
    function TsRestFrameworkApp() {
        this.pathBase = '/';
        this.express = Express();
    }
    // Configure Express middleware.
    TsRestFrameworkApp.prototype.middleware = function () {
        this.express.use(function (req, resp, next) {
            // log things
            next();
        });
        this.express.use(bodyParser.json());
        this.express.use(bodyParser.urlencoded({ extended: false }));
    };
    TsRestFrameworkApp.prototype.setAuthenticator = function (method, authPath, authenticator) {
        if (authenticator && authPath) {
            var authRouter = express.Router();
            authRouter.use(bodyParser.json());
            authRouter.use(bodyParser.urlencoded({ extended: false }));
            if (TsRestFrameworkApp.methods.indexOf(method.toLowerCase()) !== -1) {
                authRouter[method]('/', function (req, resp, next) {
                    authenticator.authenticate(req, resp, EndPointBuilder.userModel);
                });
            }
            else {
                throw new Error('Try to add auth point with unknown method value: ' + method);
            }
            this.express.use(authPath, authRouter);
        }
        if (authenticator && authenticator.authFilter) {
            EndPointBuilder.authFilters.push(authenticator.authFilter);
        }
        EndPointBuilder.defaultAuthPermission = new IsAutenticatedPermission();
    };
    TsRestFrameworkApp.prototype.setDefaultAuthPermission = function (authPermission) {
        EndPointBuilder.defaultAuthPermission = authPermission;
    };
    TsRestFrameworkApp.prototype.setDefaultPermissionFilter = function (permissionFilter) {
        EndPointBuilder.defaultPermissionFilter = permissionFilter;
    };
    TsRestFrameworkApp.prototype.setUserModel = function (userModel) {
        EndPointBuilder.userModel = userModel;
    };
    TsRestFrameworkApp.prototype.setPathBase = function (base) {
        this.pathBase = base;
        if (this.pathBase[0] !== '/') {
            this.pathBase = '/' + this.pathBase;
        }
    };
    TsRestFrameworkApp.prototype.setDbConfig = function (config) {
        this.dbConfig = config;
    };
    // Configure API endpoints.
    TsRestFrameworkApp.prototype.import = function (view, parentPath) {
        if (parentPath === void 0) { parentPath = this.pathBase; }
        if (RouteManager.getInstance().isRegisteredNamespace(view)) {
            this.importNamespace(view, this.pathBase);
        }
        else if (RouteManager.getInstance().isRegisteredViewset(view)) {
            this.importViewset(view, this.pathBase);
        }
        else {
            throw new Error('"' + view.name + '" is not registered did you miss "@Namespace" or "@Route" annotation ?');
        }
        return this;
    };
    TsRestFrameworkApp.prototype.importViewset = function (mViewset, parentPath) {
        var bundle = RouteManager.getInstance().getRouteBundle(mViewset);
        if (!bundle) {
            throw new Error('ViewSet "' + mViewset.name + '" has no configuration, did you miss "@Route" decorator on this ViewSet ?');
        }
        var router = express.Router();
        var routePath = this.appendPath(parentPath, bundle.config.path);
        if (bundle.config.middlewares && bundle.config.middlewares.length) {
            for (var _i = 0, _a = bundle.config.middlewares; _i < _a.length; _i++) {
                var requestHandler = _a[_i];
                router.use(requestHandler);
            }
        }
        else {
            router.use(bodyParser.json());
            router.use(bodyParser.urlencoded({ extended: false }));
        }
        for (var key in bundle.routes) {
            if (bundle.routes.hasOwnProperty(key)) {
                var endPointConfig = bundle.routes[key];
                this.addRoute(mViewset, key, bundle.config, endPointConfig, router);
            }
        }
        this.express.use(routePath, router);
    };
    TsRestFrameworkApp.prototype.addRoute = function (mViewset, key, routeConfig, endPointConfig, router) {
        if (!endPointConfig.path || endPointConfig.path[0] !== '/') {
            endPointConfig.path = '/' + endPointConfig.path;
        }
        endPointConfig.method = endPointConfig.method || 'get';
        var endPoint = EndPointBuilder.build(mViewset, key, routeConfig, endPointConfig, router);
        if (TsRestFrameworkApp.methods.indexOf(endPointConfig.method.toLowerCase()) !== -1) {
            router[endPointConfig.method](endPointConfig.path, endPoint);
        }
        else {
            throw new Error('Try to add end point with unknown method');
        }
    };
    TsRestFrameworkApp.prototype.importNamespace = function (mNamespace, parentPath) {
        var namespaceConfig = RouteManager.getInstance().getNamespaceConfiguration(mNamespace);
        namespaceConfig.path = namespaceConfig.path || '';
        var childPath = this.appendPath(parentPath, namespaceConfig.path);
        for (var _i = 0, _a = namespaceConfig.views; _i < _a.length; _i++) {
            var view = _a[_i];
            this.import(view, childPath);
        }
    };
    TsRestFrameworkApp.prototype.appendPath = function (parentPath, childPath) {
        var completePath;
        if (!childPath) {
            completePath = parentPath;
        }
        else if (parentPath[parentPath.length - 1] === '/' && childPath[0] === '/') {
            completePath = parentPath + childPath.substr(1, childPath.length);
        }
        else if (parentPath[parentPath.length - 1] === '/' || childPath[0] === '/') {
            completePath = parentPath + childPath;
        }
        else {
            completePath = parentPath + '/' + childPath;
        }
        return completePath;
    };
    TsRestFrameworkApp.prototype.start = function () {
        QueryManager.init(this.dbConfig);
        var port = normalizePort(process.env.PORT || 3000);
        this.express.set('port', port);
        var server = http.createServer(this.express);
        server.listen(port);
        server.on('error', onError);
        server.on('listening', onListening);
        function normalizePort(val) {
            var nPort = (typeof val === 'string') ? parseInt(val, 10) : val;
            if (isNaN(nPort)) {
                return val;
            }
            else if (nPort >= 0) {
                return nPort;
            }
            else {
                return false;
            }
        }
        function onError(error) {
            if (error.syscall !== 'listen') {
                throw error;
            }
            var bind = (typeof port === 'string') ? 'Pipe ' + port : 'Port ' + port;
            switch (error.code) {
                case 'EACCES':
                    Log.e(bind + " requires elevated privileges");
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    Log.e(bind + " is already in use");
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        }
        function onListening() {
            var addr = server.address();
            var bind = (typeof addr === 'string') ? "pipe " + addr : "port " + addr.port;
            Log.i("Listening on " + bind);
        }
    };
    TsRestFrameworkApp.methods = ['get', 'post', 'put', 'patch', 'delete', 'options', 'head', 'connect', 'trace', 'all'];
    return TsRestFrameworkApp;
}());
export { TsRestFrameworkApp };
//# sourceMappingURL=ts-rest-framework-app.js.map